//
//  itWatch.m
//  -- version 0.45 --
//
//  Created by Pigmon on Thu Oct 26 2000.
//  Last Updated by Pigmon on Thu Sep 03 2001.
//  Copyright (c) 2000,2001 Pigmon. All rights reserved.
//                     mailto:pigmon@mac.com
//
//  配付条件：　以下の項目に従う場合、かつその場合、本ソースコードを自由に複製・改変することができます。
//                ・　改変したソースコードを二次配付する場合、その旨を必ず明記して下さい。
//                  (できればバージョン番号やアプリケーション名で区別できるようにして下さい）

#import "itWatch.h"
#import "InternetTime.h"
#import "constants.h"

@implementation itWatch

//////////////////////////////////////////////////////////////////////////////////
- (void) awakeFromNib
//
// 
//////////////////////////////////////////////////////////////////////////////////
{
    NSNotificationCenter*	aDefaultCenter = [NSNotificationCenter defaultCenter];

    // Dockアイコンに描画するデフォルトの背景Imageオブジェクトを作成
    
    [(backgroundImage = [[NSImage allocWithZone:[self zone]] initWithSize:NSMakeSize(128,128)]) addRepresentation:[[[NSCustomImageRep alloc] initWithDrawSelector:@selector(drawBackgroundImage:) delegate:self] autorelease]];
    [backgroundImage setBackgroundColor:[NSColor clearColor]]; //背景の背景は透明

    // DockアイコンにセットするNSImageオブジェクトを作成
    [(source = [[NSImage allocWithZone:[self zone]] initWithSize:NSMakeSize(128,128)]) addRepresentation:[[[NSCustomImageRep alloc] initWithDrawSelector:@selector(drawSource:) delegate:self] autorelease]];
    [source setBackgroundColor:[NSColor clearColor]]; //アイコンの背景は透明

    // 時計更新用のタイマーオブジェクトを生成、0.5秒ごとにupdateTimerで時間更新をチェックする。
    mTimer = [NSTimer scheduledTimerWithTimeInterval:0.5 target:self 
                        selector:@selector(updateTimer:) userInfo:self repeats:YES];
    
    // 最後にDockアイコンを更新した時のインターネットタイム。
    // 最初は必ず更新するようにインターネットタイムではあり得ない負の値をセット
    oldIT = -1;
    
    // 最後にDockアイコンを更新した時の分
    // 最初は必ず更新するように分ではあり得ない価をセット
    oldMinute = 255;
    
    // インターネットタイムオブジェクトを生成・初期化
    mITTime = [[InternetTime alloc] init];

    // 環境設定データベースオブジェクトを作成
    mPreferences = [ NSUserDefaults standardUserDefaults ];
    
    // アプリケーション起動完了時の処理をおこなうためのNotificationを登録する。
    // アプリケーション起動完了時には ApplicationDidFinishLaunching メソッドが起動
    [aDefaultCenter addObserver:self
            selector:@selector(ApplicationDidFinishLaunching:)
            name:NSApplicationDidFinishLaunchingNotification object:nil];

    // アプリケーション終了時の処理をおこなうためのNotificationを登録する。
    // アプリケーション終了時には ApplicationWillTerminate メソッドが起動
    [aDefaultCenter addObserver:self
            selector:@selector(ApplicationWillTerminate:)
            name:NSApplicationWillTerminateNotification object:nil];

    // 環境設定変更時の処理をおこなうためのNotificationを登録する。
    // 環境設定変更時の処理時には update メソッドが起動
    [aDefaultCenter addObserver:self
            selector:@selector(update)
            name:@"PreferencesDidUpdated" object:nil];
}


//////////////////////////////////////////////////////////////////////////////////
- (void) ApplicationDidFinishLaunching:(id)notification
//
// アプリケーション起動完了後の処理
//////////////////////////////////////////////////////////////////////////////////
{
    // 環境設定のデフォルト値をセットする
    [self setDefaultPrefernces];

    // Serviceを登録する
    NSRegisterServicesProvider(self, @"itWatch");

    // 背景Imageを描画
    [backgroundImage　recache];
    
    // フローティングの時計ウィンドウを作成
    // ※　フローティングウィンドウの表示が必要になるまで作成を遅らせるよう要改良
    clockController = [ClockController alloc];
    [NSBundle loadNibNamed: @"Clock.nib" owner: clockController];
    [clockController setBackgroundImage: backgroundImage];
    [clockController setITObject: mITTime];
}


//////////////////////////////////////////////////////////////////////////////////
- (void) ApplicationWillTerminate:(id)notification
//
// Application Notificationによってアプリケーション終了時に呼び出されるメソッド
// Dockアイコンをアプリケーションのデフォルトのアイコンに戻す。（こうしないと時計として動いている
// かどうかが分かりにくい）
// 問題点：強制終了した時はここのコードは無効
//////////////////////////////////////////////////////////////////////////////////
{
    // デフォルトのアプリケーションアイコンのイメージオブジェクトを取得
    //id iconImage = [NSImage imageNamed:@"NSApplicationIcon"];
    id iconImage = [NSImage imageNamed:@"FinderIcon.icns"];
    
    // Dockアイコンをアプリケーションアイコンにする
    [NSApp setApplicationIconImage:iconImage];
}

//////////////////////////////////////////////////////////////////////////////////
- (void)drawBackgroundImage:(NSCustomImageRep *)imageRep 
//
// Dockアイコンにセットする背景のNSImageオブジェクトの描画をおこなう
//////////////////////////////////////////////////////////////////////////////////
{
    id atString= [[[NSString alloc] initWithString:@"@"] autorelease];
    id atFont = [[NSFont fontWithName:@"Helvetica-Bold" size:76] autorelease];
    id atAttributes = [[[NSMutableDictionary alloc] init] autorelease];

    id aBezier = [[[NSBezierPath alloc] init] autorelease];

    // 背景の白い四角形を描く
    [[NSColor colorWithCalibratedRed: 0.9 green: 0.9 blue: 1.0 alpha:0.3] set];
    [aBezier moveToPoint:NSMakePoint(64,0)];
    [aBezier lineToPoint:NSMakePoint(0,64)];
    [aBezier lineToPoint:NSMakePoint(64,128)];
    [aBezier lineToPoint:NSMakePoint(128,64)];
    [aBezier closePath];
    [aBezier fill];
    
    // でっかい＠を描画　
    // （上の四角と同様毎回同じ絵を描いているので、処理が冗長。←要改善項目）
    [atAttributes setObject:atFont forKey:NSFontAttributeName];
    [atAttributes setObject:[NSColor colorWithCalibratedRed: 1.0 green: 0.0 blue: 0.0 alpha:0.4] forKey:NSForegroundColorAttributeName];
    [atString drawAtPoint:NSMakePoint( 24, 45 ) withAttributes:atAttributes];
}

//////////////////////////////////////////////////////////////////////////////////
- (void)drawSource:(NSCustomImageRep *)imageRep 
//
// DockアイコンにセットするNSImageオブジェクトの描画をおこなう
//////////////////////////////////////////////////////////////////////////////////
{
    
    id	aString;
    
    id aFont = [[NSFont fontWithName:@"Helvetica-BoldOblique" size:56] autorelease];
    id aAttributes = [[[NSMutableDictionary alloc] init] autorelease];
    id aParagraphStyle = [[[NSMutableParagraphStyle alloc] init] autorelease];


    // 背景Imageを描画
    [backgroundImage drawInRect:NSMakeRect(0,0,128,128) fromRect:NSMakeRect(0,0,128,128) operation:NSCompositeCopy fraction:1.0];
    
    // インターネットタイムをあらわす３桁の数字を描画
    aString = [mITTime itStringValue];

    [aAttributes setObject:aFont forKey:NSFontAttributeName];
    [aAttributes setObject:[NSColor 
                colorWithCalibratedRed: [mPreferences floatForKey: @"fontColor1Red"]
                green: [mPreferences floatForKey: @"fontColor1Green"]
                blue: [mPreferences floatForKey: @"fontColor1Blue"]
                alpha:[mPreferences floatForKey: @"fontColor1Alpha"]]
                forKey:NSForegroundColorAttributeName];
    [aParagraphStyle setAlignment:NSCenterTextAlignment];
    [aAttributes setObject:aParagraphStyle forKey:NSParagraphStyleAttributeName];
    [aString drawInRect:NSMakeRect( 0 , 0, 120, 87 ) withAttributes:aAttributes];

    // 日付けを表示
    if ( YES == [mPreferences boolForKey: @"displayDate"] ) {
        if ( ITWATCH_PREF_DATELTZ == [mPreferences integerForKey: @"displayDateTZ"] ) {
            aString = [mITTime ltDescriptionWithCalendarFormat:@"%m.%d"];
        } else {
            aString = [mITTime itDescriptionWithCalendarFormat:@"%m.%d"];
        }
        aFont = [[NSFont fontWithName:@"Helvetica-Oblique" size:32] autorelease];
        [aAttributes setObject:aFont forKey:NSFontAttributeName];
        [aString drawInRect:NSMakeRect( 0 , 0, 120, 32 ) withAttributes:aAttributes];
    }

    // 通常時刻を表示
    if ( YES == [mPreferences boolForKey: @"displayClassicTime"] ) {
        aString = [mITTime ltDescriptionWithCalendarFormat:@"%H:%M"];
        aFont = [[NSFont fontWithName:@"Helvetica-Oblique" size:48] autorelease];
        [aAttributes setObject:aFont forKey:NSFontAttributeName];
        [aString drawInRect:NSMakeRect( 0 , 80, 128, 48 ) withAttributes:aAttributes];
    }
}


//////////////////////////////////////////////////////////////////////////////////
- (void) updateTimer:(id) aTimer
//
// 一定間隔で起動される。インターネットタイムの更新を調べる。更新されていたらDockアイコンを更新
//////////////////////////////////////////////////////////////////////////////////
{
    int		tmpIT;
    char	tmpMinute;

    // 現在のインターネットタイムを取得
    [mITTime init];
    
    // 前回更新してから、分とインターネットタイムの両方が変わっていなければ更新しない。
    tmpIT = [mITTime itIntValue];
    tmpMinute = ((long)[mITTime timeIntervalSince1970] / 60) % 60;
    if ( ( oldIT == tmpIT ) && (oldMinute == tmpMinute ) ) return;
    
    ///////////////
    // 以下は更新作業
    ///////////////
    [self update];
    
    // 現在のインターネットタイムと分の値を保存
    oldIT = tmpIT;
    oldMinute = tmpMinute;
}

//////////////////////////////////////////////////////////////////////////////////
- (void)update
//
// Dockアイコンを更新する
//////////////////////////////////////////////////////////////////////////////////
{
    // Dockアイコンにセットするイメージオブジェクトを再描画。
    // 結果として drawSource: メソッドが起動される
    [source recache];
    
    // Dockアイコンにイメージをセット
    [[NSApplication sharedApplication] setApplicationIconImage:source];
 
    // フローティングウィンドウをアップデートする
    [clockController update];
}

//////////////////////////////////////////////////////////////////////////////////
- (void) dealloc
//
// dealloc処理
//////////////////////////////////////////////////////////////////////////////////
{
    [source dealloc];
    [mTimer dealloc];
    [mITTime dealloc];
    
    [super dealloc];
}

//////////////////////////////////////////////////////////////////////////////////
- (IBAction) copyIT:(id) sender
//
// Editメニューの"Copy InternetTime"が選択された際の処理
//////////////////////////////////////////////////////////////////////////////////
{
    // デフォルトのペーストボードオブジェクトを取得
    id aPasteboard = [NSPasteboard pasteboardWithName:NSGeneralPboard];

    // 現在のインターネットタイムを取得
    [mITTime init];

    // ペーストボードのタイプを文字列に指定
    [aPasteboard declareTypes:[NSArray arrayWithObject:NSStringPboardType] owner:nil];
    
    // コピー形式の設定によってペーストボードに書き込むインターネット文字列を切り替える
    if ( ITWATCH_PREF_COPYWITHATMARK == [mPreferences integerForKey: @"copyStyle"] ) {
        // ペーストボードにインターネットタイムの文字列("@XXX"のフォーマット)をセットする
        [aPasteboard setString: [@"@" stringByAppendingString:[mITTime itStringValue]] 
                forType: NSStringPboardType ];
    } else {
        // ペーストボードにインターネットタイムの文字列("XXX"のフォーマット)をセットする
        [aPasteboard setString: [mITTime itStringValue]
                forType: NSStringPboardType ];
    }
}


//////////////////////////////////////////////////////////////////////////////////
- (void)doCopyITService:(NSPasteboard *)pboard
 userData:(NSString *)data
 error:(NSString **)error
//
// Cocoaサービス"Paste InternetTime"の処理ルーチン
//////////////////////////////////////////////////////////////////////////////////
 {
    NSArray *types;
    NSString *newString;
    types = [pboard types];

/*
    if (![types containsObject:NSStringPboardType]) {
        *error = NSLocalizedString(@"Error: Pasteboard doesn't contain a string.",
                   @"Pasteboard couldn't give string.");
        return;
    }
*/
    
    // 現在のインターネットタイムを取得
    [mITTime init];

    // コピー形式の設定によってペーストボードに書き込むインターネット文字列を切り替える
    if ( ITWATCH_PREF_COPYWITHATMARK == [mPreferences integerForKey: @"copyStyle"] ) {
        // インターネットタイムの文字列("@XXX"のフォーマット)をセットする
        newString = [[@"@" stringByAppendingString:[mITTime itStringValue]] retain];
    } else {
        // インターネットタイムの文字列("XXX"のフォーマット)をセットする
        newString = [[mITTime itStringValue] retain];
    }

    types = [NSArray arrayWithObject:NSStringPboardType];
    [pboard declareTypes:types owner:nil];
    [pboard setString:newString forType:NSStringPboardType];

    return;
}

//////////////////////////////////////////////////////////////////////////////////
- (void)setDefaultPrefernces
//
// 環境設定のデフォルト値の設定
//////////////////////////////////////////////////////////////////////////////////
{
    NSColor* defaultColor = [NSColor colorWithCalibratedRed: 0.0 green: 0.0 blue: 0.0 alpha:0.9];
    float	aFontColorRed,
                aFontColorGreen,
                aFontColorBlue,
                aFontColorAlpha;
    
    // 日付けは表示
    if ( [mPreferences stringForKey:@"displayDate"] == nil )
        [mPreferences setBool: YES forKey: @"displayDate"];
    
    // 通常時刻は非表示
    if ( [mPreferences stringForKey:@"displayClassicTime"] == nil )
        [mPreferences setBool: NO forKey: @"displayClassicTime"];

    // インターネットのコピー形式は"@"付き
    if ( [mPreferences stringForKey:@"copyStyle"] == nil )
        [mPreferences setInteger: ITWATCH_PREF_COPYWITHATMARK forKey: @"copyStyle"];

    // 日付けの表示はインターネットタイムのタイムゾーン
    if ( [mPreferences stringForKey:@"copyStyle"] == nil )
        [mPreferences setInteger: ITWATCH_PREF_DATEITTZ forKey: @"displayDateTZ"];

    // Dock内表示フォント色は黒で不透明度0.9
    [defaultColor	getRed:	&aFontColorRed
                        green:	&aFontColorGreen
                        blue:	&aFontColorBlue
                        alpha:	&aFontColorAlpha];
    if ( [mPreferences objectForKey: @"fontColor1Red"] == nil )
        [mPreferences setFloat:aFontColorRed  forKey: @"fontColor1Red"];

    if ( [mPreferences objectForKey: @"fontColor1Green"] == nil )
        [mPreferences setFloat:aFontColorGreen  forKey: @"fontColor1Green"];

    if ( [mPreferences objectForKey: @"fontColor1Blue"] == nil )
        [mPreferences setFloat:aFontColorBlue  forKey: @"fontColor1Blue"];

    if ( [mPreferences objectForKey: @"fontColor1Alpha"] == nil )
        [mPreferences setFloat:aFontColorAlpha  forKey: @"fontColor1Alpha"];
    
    // Dock外表示フォント色は黒で不透明度0.9
    [defaultColor	getRed:	&aFontColorRed
                        green:	&aFontColorGreen
                        blue:	&aFontColorBlue
                        alpha:	&aFontColorAlpha];
    if ( [mPreferences objectForKey: @"fontColor2Red"] == nil )
        [mPreferences setFloat:aFontColorRed  forKey: @"fontColor2Red"];

    if ( [mPreferences objectForKey: @"fontColor2Green"] == nil )
        [mPreferences setFloat:aFontColorGreen  forKey: @"fontColor2Green"];

    if ( [mPreferences objectForKey: @"fontColor2Blue"] == nil )
        [mPreferences setFloat:aFontColorBlue  forKey: @"fontColor2Blue"];

    if ( [mPreferences objectForKey: @"fontColor2Alpha"] == nil )
        [mPreferences setFloat:aFontColorAlpha  forKey: @"fontColor2Alpha"];
    
    // フローティングウィンドウは表示しない
    if ( [mPreferences objectForKey: @"standAloneWindow"] == nil )
        [mPreferences setBool: NO forKey: @"standAloneWindow"];
    
    // フローティングウィンドウのサイズは0.55
    if ( [mPreferences objectForKey: @"standAloneSize"] == nil )
        [mPreferences setFloat: 0.55 forKey: @"standAloneSize"];

    // フローティングウィンドウの不透明度は1.0
    if ( [mPreferences objectForKey: @"standAloneOpacity"] == nil )
        [mPreferences setFloat: 1.0 forKey: @"standAloneOpacity"];

    // フローティングウィンドウにする
    if ( [mPreferences objectForKey: @"standAloneFloating"] == nil )
        [mPreferences setBool: YES forKey: @"standAloneFloating"];
}

//////////////////////////////////////////////////////////////////////////////////
- (IBAction) showHelp:(id) sender
//
// ヘルプを表示する
//////////////////////////////////////////////////////////////////////////////////
{
    // "index.html"をバンドルからさがす。
    NSString *path = [[NSBundle mainBundle] pathForResource:@"index" ofType:@"html"];

    // "index.html"が見つかればHelp Viewerでそれを開く。
    if (path) {
        [[NSWorkspace sharedWorkspace] openFile:path withApplication: @"Help Viewer.app"];
    }　else　{
        [NSApp showHelp:self]; // 無ければNSApplicationのヘルプ表示にまかせる
    }
}
@end
